; ctls.asm:  Control Sony equipment via IR or wired Control-S port
;
; Copyright 1994, 1995 Eric Smith
;
; ctls.asm is free software; you can redistribute it and/or modify it under the
; terms of the GNU General Public License version 2 as published by the Free
; Software Foundation.  Note that I am not granting permission to redistribute
; or modify ctls.asm under the terms of any later version of the General Public
; License.
;
; This program is distributed in the hope that it will be useful (or at least
; amusing), but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License along with
; this program (in the file "COPYING"); if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; $Header: /usr/home/kolwynia/eric/pic/ctls/RCS/ctls.asm,v 1.6 1995/06/06 18:07:32 eric Exp eric $

	device	PIC16C71,XT_OSC,WDT_OFF,PROTECT_OFF

; for instruction destination argument
f	equ     1
w	equ     0

ind	equ     00h             ; used for indirect via fsr
rtcc	equ     01h             ; real time clock/counter
option	equ	81h
pcl	equ     02h             ; program counter

status	equ     3               ; status register:
cf      equ     0               ;  bit 0 = carry flag
dcf     equ     1               ;  bit 1 = digit carry flag
zf      equ     2               ;  bit 2 = zero flag
pdf     equ     3               ;  bit 3 = power down flag
tof     equ     4               ;  bit 4 = time out flag
				;  bits 5,6,7 do not apply to PIC16C54

fsr	equ     4               ; file select register (index register)

porta   equ     05h             ; I/O port A:
trisa	equ	85h
led	equ	0		;  bit 0 = LED drive - on when high
ctls	equ	1		;  bit 1 = Control-S output
serin	equ	2		;  bit 2 = RS-232 serial input
pzt     equ     3               ;  bit 3 = pzt beeper

portb   equ     6               ; I/O port B:
trisb	equ	86h		; keypad

adcon0 equ     08h
adgo    equ     2       ; go/done*
adchs0  equ     3       ; channel select 0
adchs1  equ     4       ; channel select 1
adcon1 equ     88h
adres  equ     09h
pclath equ     0ah

intcon equ     0bh
gie	equ	7	; global interrupt enable
adie	equ	6	; a/d conversion interrupt enable
rtie	equ	5	; rtcc interrupt enable
inte	equ	4	; int interrupt enable
rbie	equ	3	; port b change interrupt enable
rtif	equ	2	; rtcc interrupt flag
intf	equ	1	; int interrupt flag
rbif	equ	0	; port b change interrupt flag


; memory map
ram     equ     0ch             ; start of RAM
body    equ     000h            ; beginning of program


	org     ram             ; start of RAM

key:	ds	1

dev:	ds	1		; device code (0..31)
cmd:	ds	1		; command code (0..127)

code:	ds	2		; temp used in send (2 words)
bitcnt:	ds	1		; temp used in send


	org     body            ; start of ROM

	goto    reset           ; reset vector
	nop
	nop
	nop
	goto    reset		; interrupt vector - must be at address 4


delay12:
	goto	.+1
	goto	.+1
	nop
delay7:	goto	.+1
	nop
	return


; mapping of keys to commands

cmdmap:	andlw	00fh	; just to be safe
	addwf	pcl

	retlw	21	; power
	retlw	22	; eject
	retlw	46	; power on
	retlw	47	; power off

	retlw	24	; stop
	retlw	25	; pause
	retlw	26	; play
	retlw	27	; rewind

	retlw	28	; fast forward
	retlw	29	; record
	retlw	32	; pause engage
	retlw	89	; tape return

	retlw	90	; display on/off
	retlw	91	; open/close
	retlw	20	; x2 play
	retlw	35	; x1/5 play


; send a 40 KHz burst, cycle count in W, followed by 600 uS pause

burst:	
	bsf	porta,ctls

b1:	bsf	porta,led	; 0
	call	delay12		; 1-12
	bcf	porta,led	; 13
	call	delay7		; 14-20
	addlw	0ffh		; 21
	btfss	status,zf	; 22
	goto	b1		; 23-24

del600:	bcf	porta,ctls
	movlw	150
b2:	addlw	0ffh		; 0
	btfss	status,zf	; 1
	goto	b2		; 2-3

	return


; send one command packet based on dev and cmd

send:	movf	dev,w
	movwf	code+1
	movf	cmd,w
	addwf	cmd,w
	movwf	code
	rrf	code+1
	rrf	code

	movlw	12
	movwf	bitcnt
	movlw	96
	call	burst
s1:	movlw	24
	rrf	code+1
	rrf	code
	btfsc	status,cf
	movlw	48
s2:	call	burst
	decfsz	bitcnt
	goto	s1

; now delay to make total time at least 45 mS

	movlw	35
	movwf	bitcnt
s3:	call	del600
	decfsz	bitcnt
	goto	s3

	return


; scan one row of keypad

scanrow:
	bsf	status,cf	; assume there is a key pressed
	movwf	portb

	btfss	portb,7
	goto	sr9
	incf	key

	btfss	portb,6
	goto	sr9
	incf	key

	btfss	portb,5
	goto	sr9
	incf	key

	btfss	portb,4
	goto	sr9
	incf	key

	bcf	status,cf	; guess there wasn't
sr9:	return


; scan keypad
; if key pressed, returns with keycode in key
; if no key pressed, sleeps until there is

keyscan:
	clrf	key

	movlw	00eh
	call	scanrow
	btfsc	status,cf
	goto	ks9

	movlw	00dh
	call	scanrow
	btfsc	status,cf
	goto	ks9

	movlw	00bh
	call	scanrow
	btfsc	status,cf
	goto	ks9

	movlw	007h
	call	scanrow
	btfsc	status,cf
ks9:	return

	movlw	000h		; turn on all scan lines
	movwf	portb

	bcf	intcon,rbif	; clear the interrupt flag

	sleep
	goto	keyscan


; here we go!

reset:
	movlw	1<<serin	; set port A to outputs except serin
	tris	porta

	movlw	0f0h		; portb 7..4 are inputs (keypad return lines)
	tris	portb		; portb 3..0 are outputs (keypad scan lines)

	movlw	008h		; portb pullups enabled, prescaler on WDT
	option

	movlw	adcon1		; disable A/D to use port A as digital I/O
	movwf	fsr		;   (stupid part design - this should have
	movlw	003h		;    been the default)
	movwf	ind

	clrw
	movwf	fsr		; just to be safe

	movwf	porta		; turn off LED, control-S out, and PZT
	movwf	portb		; drive all keypad scan lines low

	movlw	(1<<rbie)	; turn on port B change interrupt
	movwf	intcon

	movlw	7		; VCR 2
	movwf	dev

;	call	beep		; debugging

main:	
	call	keyscan

	movf	key,w
	call	cmdmap
	movwf	cmd

	call	send
	goto	main

	end
